% Script file to generate figures comparable to figure 4 from:
%
% This function uses the model of TCP network flow rates from the paper
%   Joao P. Hespanha, "Stochastic Hybrid Systems: Application to
%   Communication Networks", pp.387-401, HSCC 2004, LNCS 2993.
%
% This script makes repeated calls to the function kolmogorov with a
%   variety of parameters expectFcn, whichModes and pdrop.
%
% Other parameters for the runs (grid, file distribution, etc) 
%   should be set in kolmogorov.

% Copyright 2004 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Ian Mitchell, 9/26/04

% What accuracy?
accuracy = 'medium';

% What drop rates to sample?
drops = [ 3e-4; 1e-3; 3e-3; 1e-2; 3e-2; 1e-1; 3e-1 ];
drops = [ 3e-4; 3e-3; 3e-2; 3e-1 ];
drops = [ 1e-3; 1e-2; 1e-1 ];
drops = exp(linspace(log(3e-4), log(3e-1), 13));
drops = drops(1:2:end);
numDrops = length(drops);

tMax = 30;

% What modes to sample?
modes = { 'all'; 'SS'; 'CA'; 1; 2 };
numModes = length(modes);

expectX = zeros(numModes, numDrops);
expectX2 = zeros(numModes, numDrops);

for m = 1 : numModes
  for d = 1 : numDrops

    % Find expected rate.
    data = kolmogorovTCP('mean', modes{m}, drops(d), tMax, accuracy);

    expectX(m, d) = data{end}(1);

    % Find expected rate^2.
    data = kolmogorovTCP('variance', modes{m}, drops(d), tMax, accuracy);

    expectX2(m,d) = data{end}(1);

  end
end

sdev = sqrt(expectX2 - expectX.^2);

save rates drops modes expectX expectX2 sdev


% Match the styles to Hespanha's paper.
styles = { 'k*-', 'bx-', 'r+-', 'm.-', 'c^-' };
labels = { 'total', 'SS', 'CA', 'small files', 'medium files' };

figure
for m = 1 : numModes
  hMean(m) = semilogx(drops, expectX(m, :), styles{m});
  hold on;
end
axis tight;
title('rate mean');
xlabel('drop probability');  ylabel('rate (packets/sec)');
legend(labels);
axis([ drops(1) drops(end) 0 5.4 ]);
box off;
set(hMean, 'LineWidth', 2, 'MarkerSize', 8);

figure
for m = 1 : numModes
  hSDev(m) = semilogx(drops, sdev(m, :), styles{m});
  hold on;
end
axis tight;
title('rate standard deviation');
xlabel('drop probability');  ylabel('rate (packets/sec)');
legend(labels);
axis([ drops(1) drops(end) 0 89.5 ]);
box off;
set(hSDev, 'LineWidth', 2, 'MarkerSize', 8);
