function [ data, g, data0 ] = viscousIntegrator(k1, k2, accuracy)
% viscousIntegrator: demonstrate the double integrator w/ stochastic viscosity
%
%   [ data, g, data0 ] = viscousIntegrator(k1, k2, accuracy)
%  
% In this example we calculate the expected distance from the state space
%   origin of the double integrator when it seeks to minimize that distance.
%
% To spice things up a bit, the integrator is subject to a stochastic
%   viscosity, which actually helps the input somewhat by slowing the
%   system slightly faster.
%
% This routine was written for the HSCC 2005 submission.
%
% The (stochastic) dynamics are
%
%        \dot x    = y
%	 \dot y    = b - k1 y - k2 y dB(t)
%
%   where input |b| \leq 1 is trying to stop the system at the origin,
%   and B(t) is a Brownian motion process.
%
% This function was originally designed as a script file, so most of the
%   options can only be modified in the file.
%
% For example, edit the file to change the grid dimension, boundary conditions,
%   aircraft parameters, etc.
%
% Parameters:
%
%   k1           Deterministic viscosity coefficient.  Default = 0.5
%   k2           Stochastic viscosity coefficient.  Default = 1.0
%   accuracy     Controls the order of approximations.
%                  'low'         Use odeCFL1 and upwindFirstFirst.
%                  'medium'      Use odeCFL2 and upwindFirstENO2 (default).
%                  'high'        Use odeCFL3 and upwindFirstENO3.
%                  'veryHigh'    Use odeCFL3 and upwindFirstWENO5.
%
%   data         Implicit surface function at t_max for the reach set.
%   g            Grid structure on which data was computed.
%   data0        Implicit surface function at t_0.

% Copyright 2004 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Ian Mitchell, 9/26/04

%---------------------------------------------------------------------------
% You will see many executable lines that are commented out.
%   These are included to show some of the options available; modify
%   the commenting to modify the behavior.
  
%---------------------------------------------------------------------------
% Make sure we can see the kernel m-files.
run('../../addPathToKernel');

%---------------------------------------------------------------------------
% Integration parameters.
tMax = 0.5;                  % End time.
plotSteps = 9;               % How many intermediate plots to produce?
t0 = 0;                      % Start time.
singleStep = 0;              % Plot at each timestep (overrides tPlot).

% Period at which intermediate plots should be produced.
tPlot = (tMax - t0) / (plotSteps - 1);

% How close (relative) do we need to get to tMax to be considered finished?
small = 100 * eps;

% What kind of dissipation?
dissType = 'global';

%---------------------------------------------------------------------------
% Problem Parameters.

% Bounds on the input magnitude.
inputBound = 1;

% Deterministic viscosity coefficient.
if(nargin < 1)
  viscousDeter = 0.5;
else
  viscousDeter = k1;
end

% Stochastic viscosity coefficient.
if(nargin < 2)
  viscousStoch = 1;
else
  viscousStoch = k2;
end

%---------------------------------------------------------------------------
% What level set should we view?
level = [];

% Visualize the reachable set.
displayType = 'surf';

% Pause after each plot?
pauseAfterPlot = 0;

% Delete previous plot before showing next?
deleteLastPlot = 1;

% Plot in separate subplots (set deleteLastPlot = 0 in this case)?
useSubplots = 0;

%---------------------------------------------------------------------------
% Create the grid.
g.dim = 2;
g.min = -1.1;
g.max = +1.1;
g.bdry = @addGhostExtrapolate;
g.N = 111;
g = processGrid(g);

%---------------------------------------------------------------------------
% Create initial conditions: 
%   1 - a smoothed indicator for a small circle around the origin.
radius = 0.2;
smoothing = 1/5;
signedDist = shapeSphere(g, zeros(g.dim, 1), radius);
data = 1 - 1 ./ (1 + exp(signedDist / (radius * smoothing)));
data0 = data;

%---------------------------------------------------------------------------
% Set up time approximation scheme.
integratorOptions = odeCFLset('factorCFL', 0.75, 'stats', 'on');

if(nargin < 3)
  accuracy = 'medium';
end

% Choose approximations at appropriate level of accuracy.
switch(accuracy)
 case 'low'
  derivFunc = @upwindFirstFirst;
  integratorFunc = @odeCFL1;
 case 'medium'
  derivFunc = @upwindFirstENO2;
  integratorFunc = @odeCFL2;
 case 'high'
  derivFunc = @upwindFirstENO3;
  integratorFunc = @odeCFL3;
 case 'veryHigh'
  derivFunc = @upwindFirstWENO5;
  integratorFunc = @odeCFL3;
 otherwise
  error('Unknown accuracy level %s', accuracy);
end

if(singleStep)
  integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
end

%---------------------------------------------------------------------------
% Set up spatial approximation scheme for the deterministic component.
deterministicFunc = @termLaxFriedrichs;
deterministicData.derivFunc = derivFunc;
deterministicData.hamFunc = @viscousIntegratorHamFunc;
deterministicData.partialFunc = @viscousIntegratorPartialFunc;
deterministicData.grid = g;

% The Hamiltonian and partial functions need problem parameters.
deterministicData.inputBound = inputBound;
deterministicData.viscousCoeff = viscousDeter;

% Choose degree of dissipation.
switch(dissType)
 case 'global'
  deterministicData.dissFunc = @artificialDissipationGLF;
 case 'local'
  deterministicData.dissFunc = @artificialDissipationLLF;
 case 'locallocal'
  deterministicData.dissFunc = @artificialDissipationLLLF;
 otherwise
  error('Unknown dissipation function %s', dissFunc);
end

%---------------------------------------------------------------------------
% Set up scheme for the stochastic component.

% Effect of stochastic term on dynamics.
sigma = { 0; -viscousStoch * g.xs{2} };

stochasticFunc = @termTraceHessian;
stochasticData.grid = g;
stochasticData.L = cellMatrixMultiply(sigma, cellMatrixMultiply(sigma', 0.5));
stochasticData.R = num2cell(eye(g.dim));
stochasticData.hessianFunc = @hessianSecond;

%---------------------------------------------------------------------------
% Combine the two terms.

schemeFunc = @termSum;
schemeData.innerFunc = { deterministicFunc; stochasticFunc };
schemeData.innerData = { deterministicData; stochasticData };

%---------------------------------------------------------------------------
% Initialize Display
f = figure;

% Set up subplot parameters if necessary.
if(useSubplots)
  rows = ceil(sqrt(plotSteps));
  cols = ceil(plotSteps / rows);
  plotNum = 1;
  subplot(rows, cols, plotNum);
end

h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t0) ]);

%---------------------------------------------------------------------------
% Loop until tMax (subject to a little roundoff).
tNow = t0;
startTime = cputime;
while(tMax - tNow > small * tMax)

  % Reshape data array into column vector for ode solver call.
  y0 = data(:);

  % How far to step?
  tSpan = [ tNow, min(tMax, tNow + tPlot) ];
  
  % Take a timestep.
  [ t, y, schemeData ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                               integratorOptions, schemeData);
  tNow = t(end);

  % Get back the correctly shaped data array
  data = reshape(y, g.shape);

  if(pauseAfterPlot)
    % Wait for last plot to be digested.
    pause;
  end

  % Get correct figure, and remember its current view.
  figure(f);

  % Delete last visualization if necessary.
  if(deleteLastPlot)
    delete(h);
  end

  % Move to next subplot if necessary.
  if(useSubplots)
    plotNum = plotNum + 1;
    subplot(rows, cols, plotNum);
  end

  % Create new visualization.
  h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(tNow) ]);

end

endTime = cputime;
fprintf('Total execution time %g seconds', endTime - startTime);



%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function hamValue = viscousIntegratorHamFunc(t, data, deriv, schemeData)
% viscousIntegratorHamFunc: analytic Hamiltonian for the double integrator.
%
% hamValue = viscousIntegratorHamFunc(t, data, deriv, schemeData)
%
% This function implements the hamFunc prototype for the double integrator:
%
%   hamValue = -(x_2 * deriv{1} 
%                - schemeData.inputBound * abs(deriv{2})
%                - schemeData.viscousCoeff * x_2 * deriv{2})
%
% Parameters:
%   t            Time at beginning of timestep (ignored).
%   data         Data array.
%   deriv	 Cell vector of the costate (\grad \phi).
%   schemeData	 A structure (see below).
%
%   hamValue	 The analytic hamiltonian.
%
% schemeData is a structure containing data specific to this Hamiltonian
%   For this function it contains the field(s):
%
%   .grid	    Grid structure.
%   .inputBound     Bound on the magnitude of the input.
%   .viscousCoeff  Basic amount of viscosity in the system.
%
% Ian Mitchell 9/26/04

checkStructureFields(schemeData, 'grid', 'inputBound', 'viscousCoeff');

hamValue = -(schemeData.grid.xs{2} .* deriv{1} ...
             - schemeData.inputBound .* abs(deriv{2}) ...
             - schemeData.viscousCoeff * schemeData.grid.xs{2} .* deriv{2});



%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function alpha = ...
     viscousIntegratorPartialFunc(t, data, derivMin, derivMax, schemeData, dim)
% viscousIntegratorPartialFunc: partial fcn for the double integrator.
%
%   alpha = viscousIntegratorPartialFunc(t, data, derivMin, ...
%                                       derivMax, schemeData, dim)
%
% This function implements the partialFunc prototype for the double integrator.
%
% It calculates the extrema of the absolute value of the partials of the 
%   analytic Hamiltonian with respect to the costate (gradient).
%
% Referring to the Hamiltonian from the previous function, those partials are:
%
%   in dimension 1: abs(x_2)
%   in dimension 2: schemeData.inputBound 
%                   + schemeData.viscousCoeff * abs(x_2)
%
% Parameters:
%   t            Time at beginning of timestep (ignored).
%   data         Data array.
%   derivMin	 Cell vector of minimum values of the costate (\grad \phi).
%   derivMax	 Cell vector of maximum values of the costate (\grad \phi).
%   schemeData	 A structure (see below).
%   dim          Dimension in which the partial derivatives is taken.
%
%   alpha	 Maximum absolute value of the partial of the Hamiltonian
%		   with respect to the costate in dimension dim for the 
%                  specified range of costate values (O&F equation 5.12).
%		   Note that alpha can (and should) be evaluated separately
%		   at each node of the grid.
%
% schemeData is a structure containing data specific to this Hamiltonian
%   For this function it contains the field(s):
%
%   .grid	    Grid structure.
%   .inputBound     Bound on the magnitude of the input.
%   .viscousCoeff  Basic amount of viscosity in the system.
%
% Ian Mitchell 9/26/04

checkStructureFields(schemeData, 'grid', 'inputBound', 'viscousCoeff');

switch dim
  case 1
    alpha = abs(schemeData.grid.xs{2});

  case 2
    alpha = abs(schemeData.inputBound) ...
            + schemeData.viscousCoeff * abs(schemeData.grid.xs{2});

  otherwise
    error([ 'Partials for the double integrator '...
            'only exist in dimensions 1-2' ]);
end
